package abp;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.DelayAction;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.fiacre.WaitChannel;
import obp.explorer.runtime.fiacre.FiacreQueueUtil;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.Arrays;
import java.util.ArrayList;

@SuppressWarnings("all")
public class BufferBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Idle = 1;

	private final BroadcastChannel ii;

	private final BroadcastChannel oo;

	private final WaitChannel wait0;

	private final Channel[] channels;

	public void collectChannels(List<Channel> result) {
		result.addAll(Arrays.asList(channels));
	}

	public BufferBehavior(SymbolsTable symbols, BroadcastChannel ii, BroadcastChannel oo) {
		super("buffer", idSeed++, symbols);
		this.ii = ii;
		this.oo = oo;
		inputChannels = new Channel[]{this.ii};
		outputChannels = new Channel[]{this.oo};
		symbols.registerClockIndexes(this.getId(), (short) 1);
		
		wait0 = new WaitChannel(null, symbols, this.getId(), (short) 0, 0, false, 1, false);
		channels = new Channel[] { wait0,  };
	}

	private final String[] stateNames = new String[] {"idle"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return ii.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			BufferConfiguration me = (BufferConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Idle) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final BufferConfiguration me = (BufferConfiguration) configuration.behaviorConfigurations[id];
			ii.input(id, actions);
			me.pkt = ii.value(boolean.class,0);
			if ( ii.clockId != -1 ) {
				DBM.set(configuration.dbm, ii.clockId, 0);
			}
			if (( !( (me.buff.length >= 1 ))) == true) {
			} else {
				if (true) { return false; }
			}
			me.buff = FiacreQueueUtil.enqueue(me.buff,me.pkt);
			me.state = Idle;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return oo.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			BufferConfiguration me = (BufferConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Idle) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final BufferConfiguration me = (BufferConfiguration) configuration.behaviorConfigurations[id];
			if (( !( (me.buff.length == 0))) == true) {
			} else {
				if (true) { return false; }
			}
			oo.output(id, actions,  FiacreQueueUtil.first(me.buff));
			me.buff =  FiacreQueueUtil.dequeue(me.buff);
			me.state = Idle;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return wait0.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			BufferConfiguration me = (BufferConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Idle) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final BufferConfiguration me = (BufferConfiguration) configuration.behaviorConfigurations[id];
			DBM.set(configuration.dbm, wait0.clockId, 0);
			actions.add(new DelayAction(id));
			if (( !( (me.buff.length == 0))) == true) {
			} else {
				if (true) { return false; }
			}
			me.buff =  FiacreQueueUtil.dequeue(me.buff);
			me.state = Idle;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		BufferConfiguration me = new BufferConfiguration();
		//Initial action code.
		me.state = Idle;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		BufferConfiguration me = (BufferConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case BufferBehavior.Idle: 
			text.append("@idle,");
			break;
		}
		text.append("\n\t- buff=");
		text.append(Arrays.toString(me.buff));
		text.append("\n\t- pkt=");
		text.append(Boolean.toString(me.pkt));
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		BufferConfiguration me = (BufferConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case BufferBehavior.Idle: 
			text.append("@idle,");
			break;
		}
		text.append(" buff=");
		text.append(Arrays.toString(me.buff));
		text.append(" pkt=");
		text.append(Boolean.toString(me.pkt));
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		BufferConfiguration me = new BufferConfiguration();
		me.state = boost.readShort();
		me.buff = new boolean[1];
		me.buff = new boolean[boost.readInt()];
		for (int i=0; i<me.buff.length; i++) {
			me.buff[i] = boost.readBoolean();
		}
		me.pkt = boost.readBoolean();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		BufferConfiguration me = (BufferConfiguration) processConfiguration;
		boost.writeShort(me.state);
		boost.writeInt(me.buff.length);
		for (int i=0; i<me.buff.length; i++) {
			boost.writeBoolean(me.buff[i]);
		}
		boost.writeBoolean(me.pkt);
	}

}

