package abp;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.ArrayList;

@SuppressWarnings("all")
public class ReceiverBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Rcve = 1;

	public static final short Ack = 2;

	private final BroadcastChannel mbuff;

	private final BroadcastChannel abuff;

	public ReceiverBehavior(SymbolsTable symbols, BroadcastChannel mbuff, BroadcastChannel abuff) {
		super("receiver", idSeed++, symbols);
		this.mbuff = mbuff;
		this.abuff = abuff;
		inputChannels = new Channel[]{this.mbuff};
		outputChannels = new Channel[]{this.abuff};
	}

	private final String[] stateNames = new String[] {"rcve", "ack"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return mbuff.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ReceiverConfiguration me = (ReceiverConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Rcve) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ReceiverConfiguration me = (ReceiverConfiguration) configuration.behaviorConfigurations[id];
			mbuff.input(id, actions);
			me.m = mbuff.value(boolean.class,0);
			if ( mbuff.clockId != -1 ) {
				DBM.set(configuration.dbm, mbuff.clockId, 0);
			}
			if (me.m == me.rsn) {
				me.rsn =  !(me.rsn);
				me.state = Ack;
				if (true) return true;
			} else {
				me.state = Ack;
				if (true) return true;
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return abuff.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ReceiverConfiguration me = (ReceiverConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Ack) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ReceiverConfiguration me = (ReceiverConfiguration) configuration.behaviorConfigurations[id];
			abuff.output(id, actions, me.m);
			me.state = Rcve;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		ReceiverConfiguration me = new ReceiverConfiguration();
		//Initial action code.
		me.state = Rcve;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ReceiverConfiguration me = (ReceiverConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ReceiverBehavior.Rcve: 
			text.append("@rcve,");
			break;
		case ReceiverBehavior.Ack: 
			text.append("@ack,");
			break;
		}
		text.append("\n\t- rsn=");
		text.append(Boolean.toString(me.rsn));
		text.append("\n\t- m=");
		text.append(Boolean.toString(me.m));
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ReceiverConfiguration me = (ReceiverConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ReceiverBehavior.Rcve: 
			text.append("@rcve,");
			break;
		case ReceiverBehavior.Ack: 
			text.append("@ack,");
			break;
		}
		text.append(" rsn=");
		text.append(Boolean.toString(me.rsn));
		text.append(" m=");
		text.append(Boolean.toString(me.m));
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		ReceiverConfiguration me = new ReceiverConfiguration();
		me.state = boost.readShort();
		me.rsn = boost.readBoolean();
		me.m = boost.readBoolean();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		ReceiverConfiguration me = (ReceiverConfiguration) processConfiguration;
		boost.writeShort(me.state);
		boost.writeBoolean(me.rsn);
		boost.writeBoolean(me.m);
	}

}

