package abp;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.DelayAction;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.fiacre.WaitChannel;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.Arrays;
import java.util.ArrayList;

@SuppressWarnings("all")
public class SenderBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Idle = 1;

	public static final short Send = 2;

	public static final short Waita = 3;

	private final BroadcastChannel mbuff;

	private final BroadcastChannel abuff;

	private final WaitChannel wait0;

	private final Channel[] channels;

	public void collectChannels(List<Channel> result) {
		result.addAll(Arrays.asList(channels));
	}

	public SenderBehavior(SymbolsTable symbols, BroadcastChannel mbuff, BroadcastChannel abuff) {
		super("sender", idSeed++, symbols);
		this.mbuff = mbuff;
		this.abuff = abuff;
		inputChannels = new Channel[]{this.abuff};
		outputChannels = new Channel[]{this.mbuff};
		symbols.registerClockIndexes(this.getId(), (short) 1);
		
		wait0 = new WaitChannel(null, symbols, this.getId(), (short) 0, 4, true, 5, false);
		channels = new Channel[] { wait0,  };
	}

	private final String[] stateNames = new String[] {"idle", "send", "waita"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			SenderConfiguration me = (SenderConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Idle) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final SenderConfiguration me = (SenderConfiguration) configuration.behaviorConfigurations[id];
			me.state = Waita;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return mbuff.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			SenderConfiguration me = (SenderConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Send) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final SenderConfiguration me = (SenderConfiguration) configuration.behaviorConfigurations[id];
			mbuff.output(id, actions, me.ssn);
			me.state = Waita;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return abuff.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			SenderConfiguration me = (SenderConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Waita) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final SenderConfiguration me = (SenderConfiguration) configuration.behaviorConfigurations[id];
			abuff.input(id, actions);
			me.n = abuff.value(boolean.class,0);
			if ( abuff.clockId != -1 ) {
				DBM.set(configuration.dbm, abuff.clockId, 0);
			}
			if (me.n == me.ssn) {
				me.ssn =  !(me.ssn);
			}
			me.state = Idle;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition3 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return wait0.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			SenderConfiguration me = (SenderConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Waita) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final SenderConfiguration me = (SenderConfiguration) configuration.behaviorConfigurations[id];
			DBM.set(configuration.dbm, wait0.clockId, 0);
			actions.add(new DelayAction(id));
			me.state = Send;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
		transition3,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		SenderConfiguration me = new SenderConfiguration();
		//Initial action code.
		me.state = Idle;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		SenderConfiguration me = (SenderConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case SenderBehavior.Idle: 
			text.append("@idle,");
			break;
		case SenderBehavior.Send: 
			text.append("@send,");
			break;
		case SenderBehavior.Waita: 
			text.append("@waita,");
			break;
		}
		text.append("\n\t- ssn=");
		text.append(Boolean.toString(me.ssn));
		text.append("\n\t- n=");
		text.append(Boolean.toString(me.n));
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		SenderConfiguration me = (SenderConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case SenderBehavior.Idle: 
			text.append("@idle,");
			break;
		case SenderBehavior.Send: 
			text.append("@send,");
			break;
		case SenderBehavior.Waita: 
			text.append("@waita,");
			break;
		}
		text.append(" ssn=");
		text.append(Boolean.toString(me.ssn));
		text.append(" n=");
		text.append(Boolean.toString(me.n));
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		SenderConfiguration me = new SenderConfiguration();
		me.state = boost.readShort();
		me.ssn = boost.readBoolean();
		me.n = boost.readBoolean();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		SenderConfiguration me = (SenderConfiguration) processConfiguration;
		boost.writeShort(me.state);
		boost.writeBoolean(me.ssn);
		boost.writeBoolean(me.n);
	}

}

