package bridge_none_in;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ReferencedArgumentInt;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.ArrayList;

@SuppressWarnings("all")
public class LampBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Left = 1;

	public static final short Right = 2;

	private final ReferencedArgumentInt leftRef;

	private final ReferencedArgumentInt rightRef;

	public LampBehavior(SymbolsTable symbols, ReferencedArgumentInt left, ReferencedArgumentInt right) {
		super("lamp", idSeed++, symbols);
		this.leftRef = left;
		this.rightRef = right;
	}

	private final String[] stateNames = new String[] {"left", "right"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			LampConfiguration me = (LampConfiguration) configuration.behaviorConfigurations[id];
			int left = leftRef.get(configuration);
			int right = rightRef.get(configuration);
			modifiedBehaviors[leftRef.ownerId] = true;
			modifiedBehaviors[rightRef.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			LampConfiguration me = (LampConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Left) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final LampConfiguration me = (LampConfiguration) configuration.behaviorConfigurations[id];
			int left = leftRef.get(configuration);
			int right = rightRef.get(configuration);
			try {
				if (((left) < (2)) == true) {
				} else {
					if (true) { return false; }
				}
				left = 0;
				right = 2;
				me.state = Right;
				if (true) return true;
			} finally {
				leftRef.set(configuration, left);
				rightRef.set(configuration, right);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			LampConfiguration me = (LampConfiguration) configuration.behaviorConfigurations[id];
			int right = rightRef.get(configuration);
			int left = leftRef.get(configuration);
			modifiedBehaviors[rightRef.ownerId] = true;
			modifiedBehaviors[leftRef.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			LampConfiguration me = (LampConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Right) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final LampConfiguration me = (LampConfiguration) configuration.behaviorConfigurations[id];
			int right = rightRef.get(configuration);
			int left = leftRef.get(configuration);
			try {
				if (((right) < (2)) == true) {
				} else {
					if (true) { return false; }
				}
				right = 0;
				left = 2;
				me.state = Left;
				if (true) return true;
			} finally {
				rightRef.set(configuration, right);
				leftRef.set(configuration, left);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		LampConfiguration me = new LampConfiguration();
		//Initial action code.
		me.state = Left;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		LampConfiguration me = (LampConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case LampBehavior.Left: 
			text.append("@left,");
			break;
		case LampBehavior.Right: 
			text.append("@right,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		LampConfiguration me = (LampConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case LampBehavior.Left: 
			text.append("@left,");
			break;
		case LampBehavior.Right: 
			text.append("@right,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		LampConfiguration me = new LampConfiguration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		LampConfiguration me = (LampConfiguration) processConfiguration;
		boost.writeShort(me.state);
	}

}

