package bridge_none_in;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ReferencedArgumentInt;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.ArrayList;

@SuppressWarnings("all")
public class ManBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Left = 1;

	public static final short Right = 2;

	private final BroadcastChannel d;

	private final ReferencedArgumentInt leftRef;

	private final ReferencedArgumentInt rightRef;

	public ManBehavior(SymbolsTable symbols, BroadcastChannel d, ReferencedArgumentInt left, ReferencedArgumentInt right) {
		super("man", idSeed++, symbols);
		this.d = d;
		inputChannels = new Channel[]{this.d};
		this.leftRef = left;
		this.rightRef = right;
	}

	private final String[] stateNames = new String[] {"left", "right"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return d.id;
		}

		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			ManConfiguration me = (ManConfiguration) configuration.behaviorConfigurations[id];
			int left = leftRef.get(configuration);
			modifiedBehaviors[leftRef.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ManConfiguration me = (ManConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Left) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ManConfiguration me = (ManConfiguration) configuration.behaviorConfigurations[id];
			int left = leftRef.get(configuration);
			try {
				if (((left) > (0)) == true) {
				} else {
					if (true) { return false; }
				}
				left = (left) - (1);
				d.input(id, actions);
				if ( d.clockId != -1 ) {
					DBM.set(configuration.dbm, d.clockId, 0);
				}
				me.state = Right;
				if (true) return true;
			} finally {
				leftRef.set(configuration, left);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return d.id;
		}

		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			ManConfiguration me = (ManConfiguration) configuration.behaviorConfigurations[id];
			int right = rightRef.get(configuration);
			modifiedBehaviors[rightRef.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ManConfiguration me = (ManConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Right) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ManConfiguration me = (ManConfiguration) configuration.behaviorConfigurations[id];
			int right = rightRef.get(configuration);
			try {
				if (((right) > (0)) == true) {
				} else {
					if (true) { return false; }
				}
				right = (right) - (1);
				d.input(id, actions);
				if ( d.clockId != -1 ) {
					DBM.set(configuration.dbm, d.clockId, 0);
				}
				me.state = Left;
				if (true) return true;
			} finally {
				rightRef.set(configuration, right);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		ManConfiguration me = new ManConfiguration();
		//Initial action code.
		me.state = Left;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ManConfiguration me = (ManConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ManBehavior.Left: 
			text.append("@left,");
			break;
		case ManBehavior.Right: 
			text.append("@right,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ManConfiguration me = (ManConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ManBehavior.Left: 
			text.append("@left,");
			break;
		case ManBehavior.Right: 
			text.append("@right,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		ManConfiguration me = new ManConfiguration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		ManConfiguration me = (ManConfiguration) processConfiguration;
		boost.writeShort(me.state);
	}

}

