package bridge_wait;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.DelayAction;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.fiacre.WaitChannel;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ReferencedArgumentInt;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.Arrays;
import java.util.ArrayList;

@SuppressWarnings("all")
public class Man3Behavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Left = 1;

	public static final short Right = 2;

	private final ReferencedArgumentInt leftRef;

	private final ReferencedArgumentInt rightRef;

	private final WaitChannel wait0;

	private final WaitChannel wait1;

	private final Channel[] channels;

	public void collectChannels(List<Channel> result) {
		result.addAll(Arrays.asList(channels));
	}

	public Man3Behavior(SymbolsTable symbols, ReferencedArgumentInt left, ReferencedArgumentInt right) {
		super("man3", idSeed++, symbols);
		this.leftRef = left;
		this.rightRef = right;
		symbols.registerClockIndexes(this.getId(), (short) 1);
		
		wait0 = new WaitChannel(null, symbols, this.getId(), (short) 0, 5, false);
		wait1 = new WaitChannel(null, symbols, this.getId(), (short) 0, 5, false);
		channels = new Channel[] { wait0, wait1,  };
	}

	private final String[] stateNames = new String[] {"left", "right"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return wait0.id;
		}

		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			Man3Configuration me = (Man3Configuration) configuration.behaviorConfigurations[id];
			int left = leftRef.get(configuration);
			modifiedBehaviors[leftRef.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			Man3Configuration me = (Man3Configuration) configuration.behaviorConfigurations[id];
			if (me.state != Left) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final Man3Configuration me = (Man3Configuration) configuration.behaviorConfigurations[id];
			int left = leftRef.get(configuration);
			try {
				if (((left) > (0)) == true) {
				} else {
					if (true) { return false; }
				}
				left = (left) - (1);
				DBM.set(configuration.dbm, wait0.clockId, 0);
				actions.add(new DelayAction(id));
				me.state = Right;
				if (true) return true;
			} finally {
				leftRef.set(configuration, left);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return wait1.id;
		}

		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			Man3Configuration me = (Man3Configuration) configuration.behaviorConfigurations[id];
			int right = rightRef.get(configuration);
			modifiedBehaviors[rightRef.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			Man3Configuration me = (Man3Configuration) configuration.behaviorConfigurations[id];
			if (me.state != Right) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final Man3Configuration me = (Man3Configuration) configuration.behaviorConfigurations[id];
			int right = rightRef.get(configuration);
			try {
				if (((right) > (0)) == true) {
				} else {
					if (true) { return false; }
				}
				right = (right) - (1);
				DBM.set(configuration.dbm, wait1.clockId, 0);
				actions.add(new DelayAction(id));
				me.state = Left;
				if (true) return true;
			} finally {
				rightRef.set(configuration, right);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		Man3Configuration me = new Man3Configuration();
		//Initial action code.
		me.state = Left;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		Man3Configuration me = (Man3Configuration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case Man3Behavior.Left: 
			text.append("@left,");
			break;
		case Man3Behavior.Right: 
			text.append("@right,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		Man3Configuration me = (Man3Configuration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case Man3Behavior.Left: 
			text.append("@left,");
			break;
		case Man3Behavior.Right: 
			text.append("@right,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		Man3Configuration me = new Man3Configuration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		Man3Configuration me = (Man3Configuration) processConfiguration;
		boost.writeShort(me.state);
	}

}

