package fischer;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.DelayAction;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.fiacre.WaitChannel;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ReferencedArgumentInt;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.Arrays;
import java.util.ArrayList;

@SuppressWarnings("all")
public class ProcBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short WaitLock = 1;

	public static final short WaitLock2 = 2;

	public static final short SetLock = 3;

	public static final short TestLock = 4;

	public static final short CriticalSection = 5;

	private int pid;

	private final ReferencedArgumentInt lockRef;

	private final WaitChannel wait0;

	private final WaitChannel wait1;

	private final Channel[] channels;

	public void collectChannels(List<Channel> result) {
		result.addAll(Arrays.asList(channels));
	}

	public ProcBehavior(SymbolsTable symbols, int pid, ReferencedArgumentInt lock) {
		super("Proc", idSeed++, symbols);
		this.pid = pid;
		this.lockRef = lock;
		symbols.registerClockIndexes(this.getId(), (short) 1);
		
		wait0 = new WaitChannel(null, symbols, this.getId(), (short) 0, 0, false, 2, false);
		wait1 = new WaitChannel(null, symbols, this.getId(), (short) 0, 2, true);
		channels = new Channel[] { wait0, wait1,  };
	}

	private final String[] stateNames = new String[] {"WaitLock", "WaitLock2", "SetLock", "TestLock", "CriticalSection"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			int lock = lockRef.get(configuration);
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != WaitLock) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			int lock = lockRef.get(configuration);
			try {
				if ((lock == 0) == true) {
				} else {
					if (true) { return false; }
				}
				me.state = WaitLock2;
				if (true) return true;
			} finally {
				lockRef.set(configuration, lock);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return wait0.id;
		}

		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			int lock = lockRef.get(configuration);
			modifiedBehaviors[lockRef.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != WaitLock2) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			int lock = lockRef.get(configuration);
			try {
				DBM.set(configuration.dbm, wait0.clockId, 0);
				actions.add(new DelayAction(id));
				lock = pid;
				me.state = SetLock;
				if (true) return true;
			} finally {
				lockRef.set(configuration, lock);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return wait1.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != SetLock) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			DBM.set(configuration.dbm, wait1.clockId, 0);
			actions.add(new DelayAction(id));
			me.state = TestLock;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition3 = 	new Transition(id) {
		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			int lock = lockRef.get(configuration);
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != TestLock) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			int lock = lockRef.get(configuration);
			try {
				if (lock == pid) {
					me.state = CriticalSection;
					if (true) return true;
				} else {
					me.state = WaitLock;
					if (true) return true;
				}
			} finally {
				lockRef.set(configuration, lock);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition4 = 	new Transition(id) {
		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			int lock = lockRef.get(configuration);
			modifiedBehaviors[lockRef.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != CriticalSection) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			int lock = lockRef.get(configuration);
			try {
				lock = 0;
				me.state = WaitLock;
				if (true) return true;
			} finally {
				lockRef.set(configuration, lock);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
		transition3,
		transition4,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		ProcConfiguration me = new ProcConfiguration();
		//Initial action code.
		me.state = WaitLock;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ProcConfiguration me = (ProcConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ProcBehavior.WaitLock: 
			text.append("@WaitLock,");
			break;
		case ProcBehavior.WaitLock2: 
			text.append("@WaitLock2,");
			break;
		case ProcBehavior.SetLock: 
			text.append("@SetLock,");
			break;
		case ProcBehavior.TestLock: 
			text.append("@TestLock,");
			break;
		case ProcBehavior.CriticalSection: 
			text.append("@CriticalSection,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ProcConfiguration me = (ProcConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ProcBehavior.WaitLock: 
			text.append("@WaitLock,");
			break;
		case ProcBehavior.WaitLock2: 
			text.append("@WaitLock2,");
			break;
		case ProcBehavior.SetLock: 
			text.append("@SetLock,");
			break;
		case ProcBehavior.TestLock: 
			text.append("@TestLock,");
			break;
		case ProcBehavior.CriticalSection: 
			text.append("@CriticalSection,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		ProcConfiguration me = new ProcConfiguration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		ProcConfiguration me = (ProcConfiguration) processConfiguration;
		boost.writeShort(me.state);
	}

}

