package peterson;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ReferencedArgumentInt;
import obp.explorer.runtime.ReferencedArgument;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.ArrayList;

@SuppressWarnings("all")
public class ProcBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short Idle = 1;

	public static final short Waits = 2;

	public static final short CS = 3;

	private int pid;

	private final ReferencedArgument<boolean[]> flagRef;

	private final ReferencedArgumentInt turnRef;

	public ProcBehavior(SymbolsTable symbols, int pid, ReferencedArgument<boolean[]> flag, ReferencedArgumentInt turn) {
		super("Proc", idSeed++, symbols);
		this.pid = pid;
		this.flagRef = flag;
		this.turnRef = turn;
	}

	private final String[] stateNames = new String[] {"idle", "waits", "CS"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			int turn = turnRef.get(configuration);
			boolean[] flag = flagRef.get(configuration);
			modifiedBehaviors[turnRef.ownerId] = true;
			modifiedBehaviors[flagRef.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Idle) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			int turn = turnRef.get(configuration);
			boolean[] flag = flagRef.get(configuration);
			try {
				//Start copy variable 'flag'.
				{
					boolean[] __tmp;
					__tmp = new boolean[2];
					for (int k=0; k<flag.length; k++) {
						__tmp[k] = flag[k];
					}
					flag = __tmp;
				}
				//End copy variable 'flag'.
				flag[pid] = true;
				turn = (1) - (pid);
				me.state = Waits;
				if (true) return true;
			} finally {
				turnRef.set(configuration, turn);
				flagRef.set(configuration, flag);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			int turn = turnRef.get(configuration);
			boolean[] flag = flagRef.get(configuration);
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Waits) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			int turn = turnRef.get(configuration);
			boolean[] flag = flagRef.get(configuration);
			try {
				if (( !((flag[(1) - (pid)]) && (turn == (1) - (pid)))) == true) {
				} else {
					if (true) { return false; }
				}
				me.state = CS;
				if (true) return true;
			} finally {
				turnRef.set(configuration, turn);
				flagRef.set(configuration, flag);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			boolean[] flag = flagRef.get(configuration);
			modifiedBehaviors[flagRef.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != CS) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			boolean[] flag = flagRef.get(configuration);
			try {
				//Start copy variable 'flag'.
				{
					boolean[] __tmp;
					__tmp = new boolean[2];
					for (int k=0; k<flag.length; k++) {
						__tmp[k] = flag[k];
					}
					flag = __tmp;
				}
				//End copy variable 'flag'.
				flag[pid] = false;
				me.state = Idle;
				if (true) return true;
			} finally {
				flagRef.set(configuration, flag);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		ProcConfiguration me = new ProcConfiguration();
		//Initial action code.
		me.state = Idle;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ProcConfiguration me = (ProcConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ProcBehavior.Idle: 
			text.append("@idle,");
			break;
		case ProcBehavior.Waits: 
			text.append("@waits,");
			break;
		case ProcBehavior.CS: 
			text.append("@CS,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ProcConfiguration me = (ProcConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ProcBehavior.Idle: 
			text.append("@idle,");
			break;
		case ProcBehavior.Waits: 
			text.append("@waits,");
			break;
		case ProcBehavior.CS: 
			text.append("@CS,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		ProcConfiguration me = new ProcConfiguration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		ProcConfiguration me = (ProcConfiguration) processConfiguration;
		boost.writeShort(me.state);
	}

}

