package syncorder;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ReferencedArgumentInt;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.ArrayList;

@SuppressWarnings("all")
public class InPBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short S0 = 1;

	private final BroadcastChannel cha;

	private int id_;

	private final ReferencedArgumentInt sharedRef;

	public InPBehavior(SymbolsTable symbols, BroadcastChannel cha, int id_, ReferencedArgumentInt shared) {
		super("inP", idSeed++, symbols);
		this.cha = cha;
		inputChannels = new Channel[]{this.cha};
		this.id_ = id_;
		this.sharedRef = shared;
	}

	private final String[] stateNames = new String[] {"s0"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return cha.id;
		}

		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			InPConfiguration me = (InPConfiguration) configuration.behaviorConfigurations[id];
			int shared = sharedRef.get(configuration);
			modifiedBehaviors[sharedRef.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			InPConfiguration me = (InPConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != S0) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final InPConfiguration me = (InPConfiguration) configuration.behaviorConfigurations[id];
			int shared = sharedRef.get(configuration);
			try {
				cha.input(id, actions);
				if ( cha.clockId != -1 ) {
					DBM.set(configuration.dbm, cha.clockId, 0);
				}
				shared = id_;
				me.state = S0;
				if (true) return true;
			} finally {
				sharedRef.set(configuration, shared);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		InPConfiguration me = new InPConfiguration();
		//Initial action code.
		me.state = S0;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		InPConfiguration me = (InPConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case InPBehavior.S0: 
			text.append("@s0,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		InPConfiguration me = (InPConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case InPBehavior.S0: 
			text.append("@s0,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		InPConfiguration me = new InPConfiguration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		InPConfiguration me = (InPConfiguration) processConfiguration;
		boost.writeShort(me.state);
	}

}

