package test11_time;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.DelayAction;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.fiacre.WaitChannel;
import obp.explorer.runtime.fiacre.FiacreQueueUtil;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ReferencedArgument;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.Arrays;
import java.util.ArrayList;

@SuppressWarnings("all")
public class Proc1Behavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short State1 = 1;

	public static final short State2 = 2;

	public static final short Stop = 3;

	private final ReferencedArgument<T_msg[]> inputP1Ref;

	private final ReferencedArgument<T_msg[]> inputP2Ref;

	private final WaitChannel wait0;

	private final Channel[] channels;

	public void collectChannels(List<Channel> result) {
		result.addAll(Arrays.asList(channels));
	}

	public Proc1Behavior(SymbolsTable symbols, ReferencedArgument<T_msg[]> inputP1, ReferencedArgument<T_msg[]> inputP2) {
		super("proc1", idSeed++, symbols);
		this.inputP1Ref = inputP1;
		this.inputP2Ref = inputP2;
		symbols.registerClockIndexes(this.getId(), (short) 1);
		
		wait0 = new WaitChannel(null, symbols, this.getId(), (short) 0, 2, true);
		channels = new Channel[] { wait0,  };
	}

	private final String[] stateNames = new String[] {"state1", "state2", "stop"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			Proc1Configuration me = (Proc1Configuration) configuration.behaviorConfigurations[id];
			T_msg[] inputP2 = inputP2Ref.get(configuration);
			modifiedBehaviors[inputP2Ref.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			Proc1Configuration me = (Proc1Configuration) configuration.behaviorConfigurations[id];
			if (me.state != State1) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final Proc1Configuration me = (Proc1Configuration) configuration.behaviorConfigurations[id];
			T_msg[] inputP2 = inputP2Ref.get(configuration);
			try {
				inputP2 = FiacreQueueUtil.enqueue(id, inputP2,new T_msg.A(), actions);
				me.state = State2;
				if (true) return true;
			} finally {
				inputP2Ref.set(configuration, inputP2);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return wait0.id;
		}

		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			Proc1Configuration me = (Proc1Configuration) configuration.behaviorConfigurations[id];
			T_msg[] inputP2 = inputP2Ref.get(configuration);
			modifiedBehaviors[inputP2Ref.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			Proc1Configuration me = (Proc1Configuration) configuration.behaviorConfigurations[id];
			if (me.state != State2) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final Proc1Configuration me = (Proc1Configuration) configuration.behaviorConfigurations[id];
			T_msg[] inputP2 = inputP2Ref.get(configuration);
			try {
				DBM.set(configuration.dbm, wait0.clockId, 0);
				actions.add(new DelayAction(id));
				inputP2 = FiacreQueueUtil.enqueue(id, inputP2,new T_msg.B(), actions);
				me.state = Stop;
				if (true) return true;
			} finally {
				inputP2Ref.set(configuration, inputP2);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		Proc1Configuration me = new Proc1Configuration();
		//Initial action code.
		me.state = State1;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		Proc1Configuration me = (Proc1Configuration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case Proc1Behavior.State1: 
			text.append("@state1,");
			break;
		case Proc1Behavior.State2: 
			text.append("@state2,");
			break;
		case Proc1Behavior.Stop: 
			text.append("@stop,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		Proc1Configuration me = (Proc1Configuration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case Proc1Behavior.State1: 
			text.append("@state1,");
			break;
		case Proc1Behavior.State2: 
			text.append("@state2,");
			break;
		case Proc1Behavior.Stop: 
			text.append("@stop,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		Proc1Configuration me = new Proc1Configuration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		Proc1Configuration me = (Proc1Configuration) processConfiguration;
		boost.writeShort(me.state);
	}

}

