package lgs_pilotinternal;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.fiacre.FiacreQueueUtil;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ReferencedArgument;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.ArrayList;

@SuppressWarnings("all")
public class PilotBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short S0 = 1;

	public static final short S1 = 2;

	private final ReferencedArgument<int[]> toContextRef;

	private final ReferencedArgument<int[]> toDispatcherRef;

	public PilotBehavior(SymbolsTable symbols, ReferencedArgument<int[]> toContext, ReferencedArgument<int[]> toDispatcher) {
		super("Pilot", idSeed++, symbols);
		this.toContextRef = toContext;
		this.toDispatcherRef = toDispatcher;
	}

	private final String[] stateNames = new String[] {"s0", "s1"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			PilotConfiguration me = (PilotConfiguration) configuration.behaviorConfigurations[id];
			int[] toContext = toContextRef.get(configuration);
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			PilotConfiguration me = (PilotConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != S0) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final PilotConfiguration me = (PilotConfiguration) configuration.behaviorConfigurations[id];
			int[] toContext = toContextRef.get(configuration);
			try {
				if (( !( (toContext.length == 0))) == true) {
				} else {
					if (true) { return false; }
				}
				if (( FiacreQueueUtil.first(toContext) == LGS_PilotInternalRoot.OK) == true) {
				} else {
					if (true) { return false; }
				}
				me.state = S1;
				if (true) return true;
			} finally {
				toContextRef.set(configuration, toContext);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			PilotConfiguration me = (PilotConfiguration) configuration.behaviorConfigurations[id];
			int[] toDispatcher = toDispatcherRef.get(configuration);
			modifiedBehaviors[toDispatcherRef.ownerId] = true;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			PilotConfiguration me = (PilotConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != S1) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final PilotConfiguration me = (PilotConfiguration) configuration.behaviorConfigurations[id];
			int[] toDispatcher = toDispatcherRef.get(configuration);
			try {
				if (( !( (toDispatcher.length >= 1 ))) == true) {
				} else {
					if (true) { return false; }
				}
				toDispatcher = FiacreQueueUtil.enqueue(id, toDispatcher,LGS_PilotInternalRoot.HANDLE, actions);
				me.state = S1;
				if (true) return true;
			} finally {
				toDispatcherRef.set(configuration, toDispatcher);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		PilotConfiguration me = new PilotConfiguration();
		//Initial action code.
		me.state = S0;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		PilotConfiguration me = (PilotConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case PilotBehavior.S0: 
			text.append("@s0,");
			break;
		case PilotBehavior.S1: 
			text.append("@s1,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		PilotConfiguration me = (PilotConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case PilotBehavior.S0: 
			text.append("@s0,");
			break;
		case PilotBehavior.S1: 
			text.append("@s1,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		PilotConfiguration me = new PilotConfiguration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		PilotConfiguration me = (PilotConfiguration) processConfiguration;
		boost.writeShort(me.state);
	}

}

