package onelighttempo;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.ArrayList;

@SuppressWarnings("all")
public class InterrupteurBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short S = 1;

	public static final short WaitDone = 2;

	private final BroadcastChannel goOn;

	private final BroadcastChannel done;

	public InterrupteurBehavior(SymbolsTable symbols, BroadcastChannel goOn, BroadcastChannel done) {
		super("Interrupteur", idSeed++, symbols);
		this.goOn = goOn;
		this.done = done;
		inputChannels = new Channel[]{this.done};
		outputChannels = new Channel[]{this.goOn};
	}

	private final String[] stateNames = new String[] {"S", "waitDone"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return goOn.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			InterrupteurConfiguration me = (InterrupteurConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != S) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final InterrupteurConfiguration me = (InterrupteurConfiguration) configuration.behaviorConfigurations[id];
			goOn.output(id, actions);
			me.state = WaitDone;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return done.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			InterrupteurConfiguration me = (InterrupteurConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != WaitDone) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final InterrupteurConfiguration me = (InterrupteurConfiguration) configuration.behaviorConfigurations[id];
			done.input(id, actions);
			if ( done.clockId != -1 ) {
				DBM.set(configuration.dbm, done.clockId, 0);
			}
			me.state = S;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		InterrupteurConfiguration me = new InterrupteurConfiguration();
		//Initial action code.
		me.state = S;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		InterrupteurConfiguration me = (InterrupteurConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case InterrupteurBehavior.S: 
			text.append("@S,");
			break;
		case InterrupteurBehavior.WaitDone: 
			text.append("@waitDone,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		InterrupteurConfiguration me = (InterrupteurConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case InterrupteurBehavior.S: 
			text.append("@S,");
			break;
		case InterrupteurBehavior.WaitDone: 
			text.append("@waitDone,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		InterrupteurConfiguration me = new InterrupteurConfiguration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		InterrupteurConfiguration me = (InterrupteurConfiguration) processConfiguration;
		boost.writeShort(me.state);
	}

}

