package onelighttempo;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.InformalAction;
import obp.explorer.runtime.obs.DelayAction;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.fiacre.WaitChannel;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.Arrays;
import java.util.ArrayList;

@SuppressWarnings("all")
public class LightBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short OFF = 1;

	public static final short ON = 2;

	public static final short Ack = 3;

	private final short informal0;

	private final short informal1;

	private final BroadcastChannel goOn;

	private final BroadcastChannel done;

	private final WaitChannel wait0;

	private final Channel[] channels;

	public void collectChannels(List<Channel> result) {
		result.addAll(Arrays.asList(channels));
	}

	public LightBehavior(SymbolsTable symbols, BroadcastChannel goOn, BroadcastChannel done) {
		super("Light", idSeed++, symbols);
		this.goOn = goOn;
		this.done = done;
		inputChannels = new Channel[]{this.goOn};
		outputChannels = new Channel[]{this.done};
		informal0 = symbols.getInformalId("OFF");
		informal1 = symbols.getInformalId("ON");
		symbols.registerClockIndexes(this.getId(), (short) 1);
		
		wait0 = new WaitChannel(null, symbols, this.getId(), (short) 0, 4, false, 5, false);
		channels = new Channel[] { wait0,  };
	}

	private final String[] stateNames = new String[] {"OFF", "ON", "Ack"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return goOn.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			LightConfiguration me = (LightConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != OFF) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final LightConfiguration me = (LightConfiguration) configuration.behaviorConfigurations[id];
			actions.add(new InformalAction(id, informal0));
			goOn.input(id, actions);
			if ( goOn.clockId != -1 ) {
				DBM.set(configuration.dbm, goOn.clockId, 0);
			}
			me.state = ON;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return wait0.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			LightConfiguration me = (LightConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != ON) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final LightConfiguration me = (LightConfiguration) configuration.behaviorConfigurations[id];
			actions.add(new InformalAction(id, informal1));
			DBM.set(configuration.dbm, wait0.clockId, 0);
			actions.add(new DelayAction(id));
			me.state = Ack;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition2 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Output;
		}

		@Override
		public short getSynchronisationChannelId() {
			return done.id;
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			LightConfiguration me = (LightConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != Ack) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final LightConfiguration me = (LightConfiguration) configuration.behaviorConfigurations[id];
			done.output(id, actions);
			me.state = OFF;
			if (true) return true;
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
		transition2,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		LightConfiguration me = new LightConfiguration();
		//Initial action code.
		me.state = OFF;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		LightConfiguration me = (LightConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case LightBehavior.OFF: 
			text.append("@OFF,");
			break;
		case LightBehavior.ON: 
			text.append("@ON,");
			break;
		case LightBehavior.Ack: 
			text.append("@Ack,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		LightConfiguration me = (LightConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case LightBehavior.OFF: 
			text.append("@OFF,");
			break;
		case LightBehavior.ON: 
			text.append("@ON,");
			break;
		case LightBehavior.Ack: 
			text.append("@Ack,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		LightConfiguration me = new LightConfiguration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		LightConfiguration me = (LightConfiguration) processConfiguration;
		boost.writeShort(me.state);
	}

}

