package test_loop_and_wait_with_ports;

import org.xid.basics.serializer.Boost;
import obp.explorer.runtime.obs.AtomicAction;
import obp.explorer.runtime.core.Transition;
import obp.explorer.runtime.core.Channel;
import obp.explorer.runtime.core.BroadcastChannel;
import obp.explorer.runtime.SymbolsTable;
import obp.explorer.runtime.ReferencedArgumentBoolean;
import obp.explorer.runtime.ProcessBehavior;
import obp.explorer.runtime.ExplorationContext;
import obp.explorer.runtime.DBM;
import obp.explorer.runtime.Configuration;
import obp.explorer.runtime.BehaviorConfiguration;
import obp.explorer.runtime.AbstractCallable;
import java.util.List;
import java.util.ArrayList;

@SuppressWarnings("all")
public class ProcBehavior extends ProcessBehavior {

	private static short idSeed = 1;

	public static final short State1 = 1;

	public static final short State2 = 2;

	private final BroadcastChannel tick;

	private final BroadcastChannel tock;

	private final ReferencedArgumentBoolean goRef;

	public ProcBehavior(SymbolsTable symbols, BroadcastChannel tick, BroadcastChannel tock, ReferencedArgumentBoolean go) {
		super("proc", idSeed++, symbols);
		this.tick = tick;
		this.tock = tock;
		inputChannels = new Channel[]{this.tick, this.tock};
		this.goRef = go;
	}

	private final String[] stateNames = new String[] {"state1", "state2"};

	public int getStateId(String name) {
		for (short i=0; i<stateNames.length; i++) {
			if ( stateNames[i].equals(name) ) return (i+1);
		}
		throw new IllegalArgumentException("State '"+ name +"' doesn't exist in process '"+ this.name +"'.");
	}

	public String getStateName(short id) {
		return stateNames[id-1];
	}

	private final Transition transition0 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return tock.id;
		}

		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			boolean go = goRef.get(configuration);
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != State1) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			boolean go = goRef.get(configuration);
			try {
				tock.input(id, actions);
				if ( tock.clockId != -1 ) {
					DBM.set(configuration.dbm, tock.clockId, 0);
				}
				if (go) {
					me.state = State2;
					if (true) return true;
				} else {
					me.state = State1;
					if (true) return true;
				}
			} finally {
				goRef.set(configuration, go);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition transition1 = 	new Transition(id) {
		@Override
		public short getSynchronisationType() {
			return Input;
		}

		@Override
		public short getSynchronisationChannelId() {
			return tick.id;
		}

		@Override
		public void updateModifiedBehaviors(Configuration configuration, ExplorationContext context, boolean[] modifiedBehaviors) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			boolean go = goRef.get(configuration);
		}

		@Override
		public boolean guard(Configuration configuration, ExplorationContext context) {
			ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			if (me.state != State2) return false;
			return true;
		}

		@Override
		public boolean action(final ExplorationContext context, List<AtomicAction> actions, Configuration configuration) {
			final ProcConfiguration me = (ProcConfiguration) configuration.behaviorConfigurations[id];
			boolean go = goRef.get(configuration);
			try {
				tick.input(id, actions);
				if ( tick.clockId != -1 ) {
					DBM.set(configuration.dbm, tick.clockId, 0);
				}
				if (go) {
					me.state = State1;
					if (true) return true;
				} else {
					me.state = State2;
					if (true) return true;
				}
			} finally {
				goRef.set(configuration, go);
			}
			if (true) { return true; } else { return true; }
		}

	}

;

	private final Transition[] transitions = 	new Transition[] {
		transition0,
		transition1,
	}
;

	@Override
	public Transition[] getTransitions(Configuration configuration) {
		return transitions;
	}

	@Override
	public BehaviorConfiguration createInitialConfiguration(Configuration conf) {
		ProcConfiguration me = new ProcConfiguration();
		//Initial action code.
		me.state = State1;
		return me;
	}

	public String toStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ProcConfiguration me = (ProcConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("proc: ");
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ProcBehavior.State1: 
			text.append("@state1,");
			break;
		case ProcBehavior.State2: 
			text.append("@state2,");
			break;
		}
		text.append("\n]");
		return text.toString();
	}

	public String toDotStringConfiguration(BehaviorConfiguration behaviorConfiguration) {
		ProcConfiguration me = (ProcConfiguration) behaviorConfiguration;
		StringBuilder text = new StringBuilder();
		text.append("'");
		text.append(name);
		text.append("' [");
		switch (me.state) {
		case ProcBehavior.State1: 
			text.append("@state1,");
			break;
		case ProcBehavior.State2: 
			text.append("@state2,");
			break;
		}
		text.append("]");
		return text.toString();
	}

	@Override
	public BehaviorConfiguration readConfiguration(Boost boost) {
		ProcConfiguration me = new ProcConfiguration();
		me.state = boost.readShort();
		return me;
	}

	@Override
	public void writeConfiguration(Boost boost, BehaviorConfiguration processConfiguration) {
		ProcConfiguration me = (ProcConfiguration) processConfiguration;
		boost.writeShort(me.state);
	}

}

